@@ -15,19 +15,18 @@ from random import sample
 
 
 from .clean import normalize_url, scrub_url
-from .compatibility import TLD_EXTRACTION
 from .filters import basic_filter, extension_filter, lang_filter, \
                      path_filter, spam_filter, type_filter, validate_url
 from .network import redirection_test
 from .settings import BLACKLIST
-from .urlutils import extract_domain, fix_relative_urls, is_external
+from .urlutils import extract_domain, fix_relative_urls, is_external, is_known_link
 
 
 LOGGER = logging.getLogger(__name__)
 
 FIND_LINKS_REGEX = re.compile(r'<a [^<>]+?>', re.I)
-HREFLANG_REGEX = re.compile(r'hreflang=["\']([a-z-]+)', re.I)
-LINK_REGEX = re.compile(r'href=["\']([^ ]+?)["\']', re.I)
+HREFLANG_REGEX = re.compile(r'hreflang=["\']?([a-z-]+)', re.I)
+LINK_REGEX = re.compile(r'href=["\']?([^ ]+?)(["\']|[ >])', re.I)
 
 
 def check_url(url, strict=False, with_redirects=False, language=None, with_nav=False):
@@ -74,10 +73,12 @@ def check_url(url, strict=False, with_redirects=False, language=None, with_nav=F
             raise ValueError
 
         # internationalization and language heuristics in URL
-        if language is not None:
-            if lang_filter(url, language, strict) is False:
-                LOGGER.debug('rejected, lang filter: %s', url)
-                raise ValueError
+        if (
+            language is not None
+            and lang_filter(url, language, strict) is False
+        ):
+            LOGGER.debug('rejected, lang filter: %s', url)
+            raise ValueError
 
         # split and validate
         validation_test, parsed_url = validate_url(url)
@@ -117,40 +118,38 @@ def check_url(url, strict=False, with_redirects=False, language=None, with_nav=F
 
 def sample_urls(urllist, samplesize, exclude_min=None, exclude_max=None, strict=False, verbose=False):
     '''Sample a list of URLs by domain name, optionally using constraints on their number'''
-    lastseen, urlbuffer = None, set()
+    lastseen, urlbuffer = None, []
     if verbose is True:
         logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
     else:
         logging.basicConfig(stream=sys.stdout, level=logging.ERROR)
-    for url in sorted(urllist): # key=cmp_to_key(locale.strcoll)
+    for url in sorted(urllist):  # key=cmp_to_key(locale.strcoll)
         # first basic filter
         checked = check_url(url, strict=strict, with_redirects=False)
         if checked is None:
             continue
         url, domain = checked[0], checked[1]
         # continue collection
-        if domain == lastseen:
-            urlbuffer.add(url)
-        # sample, drop, fresh start
-        else:
+        if domain != lastseen:
+            # deduplicate URL list
+            urlbuffer = list(dict.fromkeys(urlbuffer))
             # threshold for too small websites
             if exclude_min is None or len(urlbuffer) >= exclude_min:
                 # write all the buffer
                 if len(urlbuffer) <= samplesize:
                     yield from sorted(urlbuffer)
                     LOGGER.info('%s\t\turls: %s', lastseen, len(urlbuffer))
-                # or sample URLs
+                # print all or sample URLs
+                elif exclude_max is None or len(urlbuffer) <= exclude_max:
+                    yield from sorted(sample(urlbuffer, k=samplesize))
+                    LOGGER.info('%s\t\turls: %s\tprop.: %s', lastseen, len(urlbuffer), samplesize/len(urlbuffer))
                 else:
-                    # threshold for too large websites
-                    if exclude_max is None or len(urlbuffer) <= exclude_max:
-                        yield from sorted(sample(urlbuffer, samplesize))
-                        LOGGER.info('%s\t\turls: %s\tprop.: %s', lastseen, len(urlbuffer), samplesize/len(urlbuffer))
-                    else:
-                        LOGGER.info('discarded (exclude size): %s\t\turls: %s', lastseen, len(urlbuffer))
+                    LOGGER.info('discarded (exclude size): %s\t\turls: %s', lastseen, len(urlbuffer))
             else:
                 LOGGER.info('discarded (exclude size): %s\t\turls: %s', lastseen, len(urlbuffer))
-            urlbuffer = set()
-            urlbuffer.add(url)
+            urlbuffer = []
+        # register new info
+        urlbuffer.append(url)
         lastseen = domain
 
 
@@ -179,10 +178,7 @@ def extract_links(pagecontent, base_url, external_bool, language=None,
         return validlinks
     # define host reference
     if reference is None:
-        if TLD_EXTRACTION is not None:
-            reference = TLD_EXTRACTION(base_url)
-        else:
-            reference = base_url
+        reference = base_url
     # extract links
     for link in FIND_LINKS_REGEX.findall(pagecontent):
         # https://en.wikipedia.org/wiki/Hreflang
@@ -211,10 +207,13 @@ def extract_links(pagecontent, base_url, external_bool, language=None,
         if checked is None:
             continue
         # additional cleaning step?
-        newlink = re.sub(r'/\&$', '', checked[0])
+        link = re.sub(r'/\&$', '', checked[0])
         # external/internal links
-        if external_bool == is_external(newlink, reference):
-            validlinks.add(newlink)
+        if external_bool != is_external(link, reference):
+            continue
+        if is_known_link(link, validlinks) is True:
+            continue
+        validlinks.add(link)
     # return
     LOGGER.info('%s links found – %s valid links', len(candidates), len(validlinks))
     return validlinks
