@@ -11,11 +11,7 @@ import re
 
 from urllib.parse import urlparse
 
-from langcodes import Language
-try:
-    from langcodes import tag_is_valid
-except ImportError:  # Python 3.5
-    tag_is_valid = None
+from langcodes import Language, tag_is_valid
 
 from .langinfo import COUNTRY_CODES, LANGUAGE_CODES
 
@@ -24,16 +20,18 @@ LOGGER = logging.getLogger(__name__)
 
 # content filters
 WORDPRESS_CONTENT_FILTER = re.compile(r'/(?:page|seite|user|search|gallery|gall?erie|labels|archives|uploads|modules|attachment)/|/(?:tags?|schlagwort|category|cat|kategorie|kat|auth?or)/[^/]+/?$', re.IGNORECASE)
-PARAM_FILTER = re.compile(r'\.(atom|json|css|xml|js|jpg|jpeg|png|gif|tiff|pdf|ogg|mp3|m4a|aac|avi|mp4|mov|webm|flv|ico|pls|zip|tar|gz|iso|swf)\b', re.IGNORECASE)  # , re.IGNORECASE (?=[&?])
-ADULT_FILTER = re.compile(r'\b(?:adult|amateur|arsch|cams?|cash|fick|gangbang|incest|porn|sexyeroti[ck]|sexcam|swinger|xxx|bild\-?kontakte)\b', re.IGNORECASE) # live|sex|ass|orgasm|cams|
+PARAM_FILTER = re.compile(r'\.(atom|json|css|xml|js|jpg|jpeg|png|gif|tiff|pdf|ogg|mp3|m4a|aac|avi|mp4|mov|webm|flv|ico|pls|zip|tar|gz|iso|swf)\b', re.IGNORECASE)   # (?=[&?])
+ADULT_FILTER = re.compile(r'\b(?:adult|amateur|arsch|cams?|cash|fick|gangbang|incest|porn|sexyeroti[ck]|sexcam|swinger|xxx|bild\-?kontakte)\b', re.IGNORECASE)  # ass|orgasm ?
+UNSUITABLE_FILTER = re.compile(r'\b(?:add?s?|banner|doubleclick|livestream|tradedoubler)\b|/oembed\b')
+VIDEOS_FILTER = re.compile(r'\b(?:live|videos?)\b', re.IGNORECASE)
 
 # language filter
-PATH_LANG_FILTER = re.compile(r'^(?:https?://[^/]+/)([a-z]{2})([_-][a-z]{2,3})?(?:/)', re.IGNORECASE)
+PATH_LANG_FILTER = re.compile(r'(?:https?://[^/]+/)([a-z]{2})([_-][a-z]{2,3})?(?:/)', re.IGNORECASE)
 ALL_PATH_LANGS = re.compile(r'(?:/)([a-z]{2})([_-][a-z]{2})?(?:/)', re.IGNORECASE)
 HOST_LANG_FILTER = re.compile(r'https?://([a-z]{2})\.(?:[^.]+)\.(?:[^.]+)/', re.IGNORECASE)
 
 # navigation/crawls
-NAVIGATION_FILTER = re.compile(r'/(archives|auth?or|cat|category|kat|kategorie|page|schlagwort|seite|tags?|topics?|user)/', re.IGNORECASE) # ?p=[0-9]+$
+NAVIGATION_FILTER = re.compile(r'/(archives|auth?or|cat|category|kat|kategorie|page|schlagwort|seite|tags?|topics?|user)/|\?p=[0-9]+', re.IGNORECASE)
 NOTCRAWLABLE = re.compile(r'/(login|impressum|imprint)(\.[a-z]{3,4})?/?$|/login\?|/(javascript:|mailto:|tel\.?:|whatsapp:)', re.IGNORECASE)
 # |/(www\.)?(facebook\.com|google\.com|instagram\.com|twitter\.com)/
 INDEX_PAGE_FILTER = re.compile(r'.{0,5}/index(\.[a-z]{3,4})?/?$', re.IGNORECASE)
@@ -41,7 +39,7 @@ INDEX_PAGE_FILTER = re.compile(r'.{0,5}/index(\.[a-z]{3,4})?/?$', re.IGNORECASE)
 # document types
 EXTENSION_REGEX = re.compile(r'\.[a-z]{2,5}$')
 # https://en.wikipedia.org/wiki/List_of_file_formats#Web_page
-WHITELISTED_EXTENSIONS = ('.adp', '.amp', '.asp', '.aspx', '.cfm', '.cgi', '.do', '.htm', 'html', '.jsp', '.mht', '.mhtml', '.php', '.php3', '.php4', '.php5', '.phtml', '.pl', '.shtml', '.stm', '.txt', '.xhtml', '.xml')
+WHITELISTED_EXTENSIONS = ('.adp', '.amp', '.asp', '.aspx', '.cfm', '.cgi', '.do', '.htm', 'html', '.htx', '.jsp', '.mht', '.mhtml', '.php', '.php3', '.php4', '.php5', '.phtml', '.pl', '.shtml', '.stm', '.txt', '.xhtml', '.xml')
 
 # territories whitelist
 # see also: https://babel.pocoo.org/en/latest/api/languages.html
@@ -55,16 +53,15 @@ LANGUAGE_MAPPINGS = {
 
 def basic_filter(url):
     '''Filter URLs based on basic formal characteristics'''
-    if not url.startswith('http') or len(url) >= 500 or len(url) < 10:
-        return False
-    return True
+    return bool(url.startswith('http') and 10 <= len(url) < 500)
 
 
 def extension_filter(urlpath):
     '''Filter based on file extension'''
-    if EXTENSION_REGEX.search(urlpath) and not urlpath.endswith(WHITELISTED_EXTENSIONS):
-        return False
-    return True
+    return bool(
+        not EXTENSION_REGEX.search(urlpath)
+        or urlpath.endswith(WHITELISTED_EXTENSIONS)
+    )
 
 
 def langcodes_score(language, segment, score):
@@ -75,7 +72,7 @@ def langcodes_score(language, segment, score):
     if segment[:2] not in COUNTRY_CODES and segment[:2] not in LANGUAGE_CODES:
         return score
     # test if tag is valid (caution: private codes are)
-    if tag_is_valid is None or tag_is_valid(segment):
+    if tag_is_valid(segment):
         # try to identify language code
         identified = Language.get(segment).language
         # see if it matches
@@ -97,7 +94,7 @@ def lang_filter(url, language=None, strict=False):
     # init score
     score = 0
     # first test: internationalization in URL path
-    match = PATH_LANG_FILTER.search(url)
+    match = PATH_LANG_FILTER.match(url)
     if match:
         # look for other occurrences
         occurrences = ALL_PATH_LANGS.findall(url)
@@ -118,9 +115,7 @@ def lang_filter(url, language=None, strict=False):
             else:
                 score -= 1
     # determine test result
-    if score < 0:
-        return False
-    return True
+    return score >= 0
 
 
 def path_filter(urlpath, query):
@@ -139,37 +134,27 @@ def spam_filter(url):
     #for exp in (''):
     #    if exp in url:
     #        return False
-    if ADULT_FILTER.search(url):
-        return False
-    # default
-    return True
+    return not ADULT_FILTER.search(url)
 
 
 def type_filter(url, strict=False, with_nav=False):
     '''Make sure the target URL is from a suitable type (HTML page with primarily text)'''
-    # directory
-    #if url.endswith('/'):
-    #    return False
     try:
         # feeds
         if url.endswith(('/feed', '/rss')):
             raise ValueError
-        # embedded content
-        if re.search(r'/oembed\b', url, re.IGNORECASE):
-            raise ValueError
         # wordpress structure
-        if WORDPRESS_CONTENT_FILTER.search(url):
-            if with_nav is not True or not is_navigation_page(url):
-                raise ValueError
+        if WORDPRESS_CONTENT_FILTER.search(url) and (
+            with_nav is not True or not is_navigation_page(url)
+        ):
+            raise ValueError
         # hidden in parameters
         if strict is True and PARAM_FILTER.search(url):
             raise ValueError
-        # not suitable
-        if re.match(r'https?://banner\.|https?://add?s?\.', url, re.IGNORECASE):
+        # not suitable, ads and embedded content
+        if UNSUITABLE_FILTER.search(url):
             raise ValueError
-        if re.search(r'\b(?:doubleclick|tradedoubler|livestream)\b|/(?:live|videos?)/', url, re.IGNORECASE):
-            raise ValueError
-        if strict is True and re.search(r'\b(?:live|videos?)\b', url, re.IGNORECASE):
+        if strict is True and VIDEOS_FILTER.search(url):
             raise ValueError
     except ValueError:
         return False
@@ -183,13 +168,14 @@ def validate_url(url):
         parsed_url = urlparse(url)
     except ValueError:
         return False, None
-    if bool(parsed_url.scheme) is False or parsed_url.scheme not in ('http', 'https'):
+    if not bool(parsed_url.scheme) or parsed_url.scheme not in (
+        'http',
+        'https',
+    ):
         return False, None
     if len(parsed_url.netloc) < 5 or \
        (parsed_url.netloc.startswith('www.') and len(parsed_url.netloc) < 8):
         return False, None
-    # if validators.url(parsed_url.geturl(), public=True) is False:
-    #    return False
     # default
     return True, parsed_url
 
