@@ -5,26 +5,26 @@ Unit tests for the courlan package.
 ## This file is available from https://github.com/adbar/courlan
 ## under GNU GPL v3 license
 
+import io
 import logging
 import os
+import subprocess
 import sys
+import tempfile
 
+from contextlib import redirect_stdout
 from unittest.mock import patch
 
 import pytest
 
-try:
-    import tldextract
-    TLD_EXTRACTION = tldextract.TLDExtract(suffix_list_urls=None)
-except ImportError:
-    TLD_EXTRACTION = None
-
+from courlan import cli
 from courlan import clean_url, normalize_url, scrub_url, check_url, is_external, sample_urls, validate_url, extract_links, extract_domain, fix_relative_urls, get_base_url, get_host_and_path, get_hostinfo, is_navigation_page, is_not_crawlable, lang_filter
-from courlan.cli import parse_args
 from courlan.filters import extension_filter, path_filter, spam_filter, type_filter
+from courlan.urlutils import is_known_link
 
 
 logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
+RESOURCES_DIR = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'data')
 
 
 
@@ -72,6 +72,9 @@ def test_scrub():
     # garbled URLs e.g. due to quotes
     assert scrub_url('https://www.test.com/"' + '<p></p>'*100) == 'https://www.test.com'
     assert scrub_url('https://www.test.com/"' * 50) != 'https://www.test.com'
+    # simply too long, left untouched
+    my_url = 'https://www.test.com/' + 'abcdefg' * 100
+    assert scrub_url(my_url) == my_url
 
 
 def test_extension_filter():
@@ -112,7 +115,7 @@ def test_type_filter():
     # -video- vs. /video/
     assert type_filter('http://my-videos.com/') is True
     assert type_filter('http://my-videos.com/', strict=True) is False
-    assert type_filter('http://example.com/video/1') is False
+    assert type_filter('http://example.com/video/1', strict=True) is False
     assert type_filter('http://example.com/new-video-release') is True
     assert type_filter('http://example.com/new-video-release', strict=True) is False
     # tags
@@ -122,8 +125,8 @@ def test_type_filter():
     assert type_filter('http://www.example.org/author/abcde') is False
     assert type_filter('http://www.example.org/autor/abcde/') is False
     # misc
-    # assert type_filter('http://www.bmbwk.gv.at/forschung/fps/gsk/befragung.xml?style=text') is True
-    # assert type_filter('http://www.aec.at/de/archives/prix_archive/prix_projekt.asp?iProjectID=11118') is True
+    assert type_filter('http://www.bmbwk.gv.at/forschung/fps/gsk/befragung.xml?style=text') is True
+    #assert type_filter('http://www.aec.at/de/archives/prix_archive/prix_projekt.asp?iProjectID=11118') is True
 
 
 def test_path_filter():
@@ -169,8 +172,10 @@ def test_lang_filter():
 def test_navigation():
     assert is_navigation_page('https://test.org/') is False
     assert is_navigation_page('https://test.org/page/1') is True
+    assert is_navigation_page('https://test.org/?p=11') is True
     assert is_not_crawlable('https://test.org/login') is True
     assert is_not_crawlable('https://test.org/login/') is True
+    assert is_not_crawlable('https://test.org/login.php') is True
     assert is_not_crawlable('https://test.org/page') is False
 
 
@@ -219,9 +224,6 @@ def test_urlcheck():
     #assert check_url('https://www.httpbin.org/status/302', with_redirects=True) == ('https://www.httpbin.org/status/302', 'httpbin.org')
     assert check_url('https://www.httpbin.org/status/404', with_redirects=True) is None
     assert check_url('https://www.ht.or', with_redirects=True) is None
-    if TLD_EXTRACTION is None:
-        assert check_url('http://www.example') is None
-        assert check_url('http://example.invalid/', False) is None
     # recheck type and spam filters
     assert check_url('http://example.org/code/oembed/') is None
     assert check_url('http://cams.com/', strict=False) == ('http://cams.com', 'cams.com')
@@ -257,9 +259,18 @@ def test_urlutils():
     '''Test URL manipulation tools'''
     assert extract_domain('https://httpbin.org/') == 'httpbin.org'
     assert get_base_url('https://example.org/path') == 'https://example.org'
+    with pytest.raises(ValueError):
+        assert get_host_and_path('123') is None
     assert get_host_and_path('https://example.org/path') == ('https://example.org', '/path')
-    assert get_hostinfo('https://example.org/path') == ('example.org', 'https://example.org')
     assert get_hostinfo('https://httpbin.org/') == ('httpbin.org', 'https://httpbin.org')
+    assert get_hostinfo('https://example.org/path') == ('example.org', 'https://example.org')
+    # keeping track of known URLs
+    known_links = {'https://test.org'}
+    assert is_known_link('https://test.org/1', known_links) is False
+    assert is_known_link('https://test.org', known_links) is True
+    assert is_known_link('http://test.org', known_links) is True
+    assert is_known_link('http://test.org/', known_links) is True
+    assert is_known_link('https://test.org/', known_links) is True
 
 
 def test_external():
@@ -271,22 +282,23 @@ def test_external():
     assert is_external('https://google.com/', 'https://www.google.co.uk/', ignore_suffix=False) is True
     # malformed URLs
     assert is_external('h1234', 'https://www.google.co.uk/', ignore_suffix=True) is True
-    #if TLD_EXTRACTION is not None:
-    #    # tldextract object
-    #    tldinfo = TLD_EXTRACTION('http://127.0.0.1:8080/test/')
-    #    assert is_external('https://127.0.0.1:80/', tldinfo) is False
 
 
 def test_extraction():
     '''test link comparison in HTML'''
     assert len(extract_links(None, 'https://test.com/', False)) == 0
     assert len(extract_links('', 'https://test.com/', False)) == 0
+    # link known under another form
+    pagecontent = '<html><a href="https://test.org/example"/><a href="https://test.org/example/&"/></html>'
+    assert len(extract_links(pagecontent, 'https://test.org', False)) == 1
     # language
     pagecontent = '<html><a href="https://test.com/example" hreflang="de-DE"/></html>'
     assert len(extract_links(pagecontent, 'https://test.com/', False)) == 1
     assert len(extract_links(pagecontent, 'https://test.com/', True)) == 0
     assert len(extract_links(pagecontent, 'https://test.com/', False, language='de')) == 1
     assert len(extract_links(pagecontent, 'https://test.com/', False, language='en')) == 0
+    pagecontent = '<html><a href=https://test.com/example hreflang=de-DE/></html>'
+    assert len(extract_links(pagecontent, 'https://test.com/', False, language='de')) == 1
     # x-default
     pagecontent = '<html><a href="https://test.com/example" hreflang="x-default"/></html>'
     assert len(extract_links(pagecontent, 'https://test.com/', False, language='de')) == 1
@@ -306,28 +318,73 @@ def test_extraction():
     pagecontent = '<html><a href="{privacy}" target="_privacy">{privacy-link}</a></html>'
     assert len(extract_links(pagecontent, 'https://test.com/', False)) == 0
     assert len(extract_links(pagecontent, 'https://test.com/', True)) == 0
+    # links without quotes
+    pagecontent = '<html><a href=/contact>Link</a></html>'
+    assert extract_links(pagecontent, 'https://test.com/', False) == {'https://test.com/contact'}
+    assert extract_links(pagecontent, 'https://test.com/', True) == set()
+    pagecontent = '<html><a href=/contact attribute=value>Link</a></html>'
+    assert extract_links(pagecontent, 'https://test.com/', False) == {'https://test.com/contact'}
 
 
 def test_cli():
     '''test the command-line interface'''
-    testargs = ['', '-i', 'input.txt', '--outputfile', 'output.txt', '-v', '--language', 'en']
+    testargs = ['', '-i', 'input.txt', '-d', 'discardedfile.txt', '--outputfile', 'output.txt', '-v', '--language', 'en']
     with patch.object(sys, 'argv', testargs):
-        args = parse_args(testargs)
+        args = cli.parse_args(testargs)
     assert args.inputfile == 'input.txt'
+    assert args.discardedfile == 'discardedfile.txt'
     assert args.outputfile == 'output.txt'
     assert args.verbose is True
     assert args.language == 'en'
     assert os.system('courlan --help') == 0  # exit status
 
+    # testfile
+    inputfile = os.path.join(RESOURCES_DIR, 'input.txt')
+    os_handle, temp_outputfile = tempfile.mkstemp(suffix='.txt', text=True)
+    env = os.environ.copy()
+    # Force encoding to utf-8 for Windows (seem to be a problem only in GitHub Actions)
+    if os.name == 'nt':
+        env['PYTHONIOENCODING'] = 'utf-8'
+        courlan_bin = os.path.join(sys.prefix, "Scripts", "courlan")
+    else:
+        courlan_bin = 'courlan'
+    # test for Windows and the rest
+    assert subprocess.run([courlan_bin, '-i', inputfile, '-o', temp_outputfile], env=env).returncode == 0
+
+    # tests without Windows
+    if os.name != 'nt':
+        # dry runs (writes to /tmp/)
+        testargs = ['', '-i', inputfile, '-d', '/tmp/tralala1.txt', '-o', temp_outputfile, '--language', 'en', '--strict']
+        f = io.StringIO()
+        with patch.object(sys, 'argv', testargs):
+            args = cli.parse_args(testargs)
+        with redirect_stdout(f):
+            cli.process_args(args)
+        assert len(f.getvalue()) == 0
+        testargs = ['', '-i', inputfile, '-o', '/tmp/tralala.txt', '--sample']
+        with patch.object(sys, 'argv', testargs):
+            args = cli.parse_args(testargs)
+        with redirect_stdout(f):
+            cli.process_args(args)
+        assert len(f.getvalue()) == 0
+    # delete temporary output file
+    try:
+        os.remove(temp_outputfile)
+    except PermissionError:
+        print("couldn't delete temp file")
+
 
 def test_sample():
     '''test URL sampling'''
-    assert len(list(sample_urls(['http://test.org/test1', 'http://test.org/test2'], 0))) == 0
+    assert not list(
+        sample_urls(['http://test.org/test1', 'http://test.org/test2'], 0)
+    )
+
     # assert len(sample_urls(['http://test.org/test1', 'http://test.org/test2'], 1)) == 1
     mylist = ['http://t.o/t1', 'http://test.org/test1', 'http://test.org/test2', 'http://test2.org/test2']
     assert len(list(sample_urls(mylist, 1, verbose=True))) == 1
-    assert len(list(sample_urls(mylist, 1, exclude_min=10, verbose=True))) == 0
-    assert len(list(sample_urls(mylist, 1, exclude_max=1, verbose=True))) == 0
+    assert not list(sample_urls(mylist, 1, exclude_min=10, verbose=True))
+    assert not list(sample_urls(mylist, 1, exclude_max=1, verbose=True))
 
 
 def test_examples():
